<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM - Open Source CRM application.
 * Copyright (C) 2014  Yuri Kuznetsov, Taras Machyshyn, Oleksiy Avramenko
 * Website: http://www.espocrm.com
 *
 * EspoCRM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EspoCRM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EspoCRM. If not, see http://www.gnu.org/licenses/.
 ************************************************************************/

class AfterUpgrade
{
    public function run($container)
    {
        $entityManager = $container->get('entityManager');
        $config = $container->get('config');

        function mcryptDecryptOld($encryptedString, $key) {
            $encryptedString = base64_decode($encryptedString);
            $string = substr($encryptedString, 0, strlen($encryptedString) - 16);
            $iv = substr($encryptedString, -16);

            return trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key, $string, MCRYPT_MODE_CBC, $iv));
        }

        function mcryptEncrypt($string, $key, $iv) {
            $block = mcrypt_get_block_size(MCRYPT_RIJNDAEL_128, 'cbc');
            $pad = $block - (strlen($string) % $block);
            $string .= str_repeat(chr($pad), $pad);
            return base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $key, $string, MCRYPT_MODE_CBC, $iv) . $iv);
        }

        function opensslDecrypt($encryptedString, $key) {
            $encryptedString = base64_decode($encryptedString);
            $string = substr($encryptedString, 0, strlen($encryptedString) - 16);
            $iv = substr($encryptedString, -16);

            return trim(openssl_decrypt($string, 'aes-256-cbc', $key, OPENSSL_RAW_DATA, $iv));
        }

        function mcryptDecrypt($encryptedString, $key) {
            $encryptedString = base64_decode($encryptedString);
            $string = substr($encryptedString, 0, strlen($encryptedString) - 16);
            $iv = substr($encryptedString, -16);

            $string = mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key, $string, MCRYPT_MODE_CBC, $iv);

            $block = mcrypt_get_block_size(MCRYPT_RIJNDAEL_128, 'cbc');
            $len = strlen($string);
            $pad = ord($string[$len - 1]);
            return substr($string, 0, strlen($string) - $pad);
        }

        $iv = mcrypt_create_iv(16, MCRYPT_RAND);

        $cryptKey = $config->get('cryptKey');

        if (!$cryptKey) {
            $newCryptKey = md5(uniqid());
            $config->set('cryptKey', $newCryptKey);
            $config->save();
        } else {
            $newCryptKey = $cryptKey;
        }

        $key = hash('sha256', $cryptKey, true);
        $newKey = hash('sha256', $newCryptKey, true);

        foreach ($entityManager->getRepository('EmailAccount')->find() as $entity) {
            foreach (['password', 'smtpPassword'] as $attribute) {
                $value = $entity->get($attribute);
                if (empty($value)) continue;
                $valueDecrypted = mcryptDecryptOld($value, $key);
                $valueEncrypted = mcryptEncrypt($valueDecrypted, $newKey, $iv);

                $entity->set($attribute, $valueEncrypted);
            }
            $entityManager->saveEntity($entity);
        }

        foreach ($entityManager->getRepository('InboundEmail')->find() as $entity) {
            foreach (['password'] as $attribute) {
                $value = $entity->get($attribute);
                if (empty($value)) continue;
                $valueDecrypted = mcryptDecryptOld($value, $key);
                $valueEncrypted = mcryptEncrypt($valueDecrypted, $newKey, $iv);

                $entity->set($attribute, $valueEncrypted);
            }
            $entityManager->saveEntity($entity);
        }

        foreach ($entityManager->getRepository('User')->find() as $user) {
            $entity = $entityManager->getEntity('Preferences', $user->id);
            if (!$entity) continue;
            foreach (['smtpPassword'] as $attribute) {
                $value = $entity->get($attribute);
                if (empty($value)) continue;
                $valueDecrypted = mcryptDecryptOld($value, $key);
                $valueEncrypted = mcryptEncrypt($valueDecrypted, $newKey, $iv);

                $entity->set($attribute, $valueEncrypted);
            }
            $entityManager->saveEntity($entity);
        }

    }
}
